#!/usr/bin/env tclsh

# (C) 2007-2022 Zbigniew Baniewski

# l - list, m - supposed "list member"
proc in { l m } { return [expr {([lsearch $l $m] < 0) ? 0 : 1}] }
proc ni { l m } { return [expr {([lsearch $l $m] < 0) ? 1 : 0}] }

proc balloon {w help} {
  bind $w <Any-Enter> "after 1500 [list balloon:show %W [list $help]]"
  bind $w <Any-Leave> "destroy %W.balloon"
}

proc balloon:show {w arg} {
  set fontSize [expr {$::fontSize($::appSize) - 2}]
  if {[eval winfo containing  [winfo pointerxy .]]!=$w} {return}
  set top $w.balloon
  catch {destroy $top}
  toplevel $top -bd 1 -bg black
  wm overrideredirect $top 1
  if {$::tcl_platform(platform) == "macintosh"} {
    unsupported1 style $top floating sideTitlebar
  }
  pack [message $top.txt -aspect 10000 -bg lightyellow -fg black -font "fixed -$fontSize" -text $arg]
  set wmx [winfo rootx $w]
  set wmy [expr {[winfo rooty $w]+[winfo height $w]}]
  wm geometry $top [winfo reqwidth $top.txt]x[winfo reqheight $top.txt]+$wmx+$wmy
  raise $top
}

# EKB's "Choose first available font" little procedure
proc firstFont {fontlist default} {
  set avail {}
  foreach f [font families] { lappend avail [string tolower $f] }
    foreach try $fontlist { if { [in $avail [string tolower $try]] } {
      return $try
    }
  }
  return $default
}

proc appHelp { {fullPath2file} } {
  global appSize fontFamily ySbarWidth

  if { [file exists $fullPath2file] && [file isfile $fullPath2file] && [file readable $fullPath2file] } {
    set rmh [open $fullPath2file]
    fconfigure $rmh -encoding utf-8
    set readmeText [read $rmh]
    close $rmh
  } else {
    set readmeText "[::msgcat::mc errFileNotFound [file tail $fullPath2file]]"
  }

  if { [winfo exists .hlpwin] } { return 0 }
  toplevel .hlpwin
  frame .hlpwin.top
  label .hlpwin.bottom -font $fontFamily -text "[::msgcat::mc aboutMsg]"
  pack .hlpwin.top -expand yes -fill both
  pack .hlpwin.bottom -expand no -fill x
  text .hlpwin.top.txtwin -width 75 -height 25 -font "$fontFamily -$::fontSize($::appSize)" -wrap word \
    -background white -foreground black -yscrollcommand ".hlpwin.top.sby set"
  scrollbar .hlpwin.top.sby -command ".hlpwin.top.txtwin yview" -width $ySbarWidth($appSize)
  pack .hlpwin.top.sby -side right -fill y
  pack .hlpwin.top.txtwin -side right -expand yes -fill both

  wm title .hlpwin "[::msgcat::mc aboutTitle]"
  .hlpwin.top.txtwin insert 1.0 $readmeText
  .hlpwin.top.txtwin mark set insert 1.0
  .hlpwin.top.txtwin configure -state disabled
  bind .hlpwin <Escape> {destroy .hlpwin}
  focus .hlpwin.top.txtwin
}

proc placeKnobs {deviceID rowNumber fillWindow} {
  set comment [set [join [list "::device:$deviceID" "(name)"] ""]]
  if { [string index [set [join [list "::device:$deviceID" "(access)"] ""]] 1] ne "w" } {
    set state "disabled"
    append comment " [::msgcat::mc devNoWriteAccess]"
  } elseif { [string index [set [join [list "::device:$deviceID" "(access)"] ""]] 2] eq "i" } {
    set state "disabled"
    append comment " [::msgcat::mc devInactive]"
  } elseif { [string index [set [join [list "::device:$deviceID" "(access)"] ""]] 4] eq "l" } {
    set state "disabled"
    append comment " [::msgcat::mc devLocked]"
  } else {
    set [join [list "::device:$deviceID" "(operational)"] ""] yes
    set state "normal"
  }
  if { [set [join [list "::device:$deviceID" "(knobs)"] ""]] > 1 } {
    set tmplist [split [set [join [list "::device:$deviceID" "(values)"] ""]] ","]
    set ii 0
    foreach tmpval $tmplist {
      set ::v$deviceID,$ii $tmpval
      incr ii
    }
  } else {
    set tmplist [set [join [list "::device:$deviceID" "(values)"] ""]]
    set ::v$deviceID,0 [lindex $tmplist 0]
  }
  if { [string first "Playback" [set [join [list "::device:$deviceID" "(name)"] ""]] ] >= 0 } {
    set FrameBGColour green
    set FrameFGColour black
  } elseif { [string first "Capture" [set [join [list "::device:$deviceID" "(name)"] ""]] ] >= 0 } {
    set FrameBGColour red
    set FrameFGColour black
  } else {
    set FrameBGColour blue
    set FrameFGColour white
  }



  if { $fillWindow } {


    if { [in [list INTEGER BOOLEAN ENUMERATED IEC958] [set [join [list "::device:$deviceID" "(type)"] ""]]] } {
      frame $::mainf.frame$deviceID
      grid $::mainf.frame$deviceID -row $rowNumber -column 1 -sticky ew
      grid columnconfigure $::mainf 1 -weight 1
      label $::mainf.label$deviceID -text "$deviceID" -background $FrameBGColour -foreground $FrameFGColour
      if {$::appSize eq "small"} {balloon $::mainf.label$deviceID $comment}
      grid $::mainf.label$deviceID -row $rowNumber -column 0
#      frame $::mainf.sideframe$deviceID
    }

    if { [set [join [list "::device:$deviceID" "(type)"] ""]] eq "INTEGER" } {
      set ii 0
      for {set ii 0} {$ii < [llength $tmplist]} {incr ii} {
        scale $::mainf.frame$deviceID.knob$ii -from [set [join [list "::device:$deviceID" "(min)"] ""]] -to [set [join [list "::device:$deviceID" "(max)"] ""]] \
          -length $::scaleLength($::appSize) -width $::sliderWidth($::appSize) -orient horizontal -variable [subst {::v$deviceID,$ii}] -showvalue false -state $state -command "moveSlider $deviceID $ii"
        pack $::mainf.frame$deviceID.knob$ii -expand 1 -fill x
      }
      incr ii -1
      if {$::appSize ne "small"} {
        $::mainf.frame$deviceID.knob$ii configure -tickinterval 2500
        foreach {maxVal tickInt} {6 1 51 5 101 10 501 50 1001 100 5001 500 10001 1000} {
          if { [set [join [list "::device:$deviceID" "(max)"] ""]] < $maxVal } {
            $::mainf.frame$deviceID.knob$ii configure -tickinterval $tickInt
            break
          }
        }
      }
      set ii 0
      if {$::appSize ne "small"} {
        $::mainf.frame$deviceID.knob$ii configure -label $comment
      }
    } elseif { [set [join [list "::device:$deviceID" "(type)"] ""]] eq "IEC958" } {
      if {[llength $tmplist]} {
        set ii 0
        for {set ii 0} {$ii < [llength $tmplist]} {incr ii} {
          label $::mainf.frame$deviceID.knob$ii -text [expr {($ii == [llength $tmplist] - 1) ? $comment : ""}]
          pack $::mainf.frame$deviceID.knob$ii -side left
        }
      } else {
        label $::mainf.frame$deviceID.knob0 -text $comment
        pack $::mainf.frame$deviceID.knob0 -side left
      }
    } elseif { [set [join [list "::device:$deviceID" "(type)"] ""]] eq "BOOLEAN" } {
      set ii 0
      for {set ii 0} {$ii < [llength $tmplist]} {incr ii} {
        checkbutton $::mainf.frame$deviceID.knob$ii -variable [subst {::v$deviceID,$ii}] -offvalue off -onvalue on -text [expr {($ii == [llength $tmplist] - 1) ? $comment : ""}] -command "pressButton $deviceID $ii" -state $state
        pack $::mainf.frame$deviceID.knob$ii -side left
      }
    } elseif { [set [join [list "::device:$deviceID" "(type)"] ""]] eq "ENUMERATED" } {
      frame $::mainf.frame$deviceID.fr0
      frame $::mainf.frame$deviceID.fr1
      frame $::mainf.frame$deviceID.fr2
      pack $::mainf.frame$deviceID.fr0 $::mainf.frame$deviceID.fr1 $::mainf.frame$deviceID.fr2 -side left
      if {$::appSize ne "small"} {
        label $::mainf.frame$deviceID.fr0.l -text $comment
        pack $::mainf.frame$deviceID.fr0.l
      }
      set ii 0
      set where "fr1"
      for {set ii 0} {$ii < [llength [set [join [list "::device:$deviceID" "(items)"] ""]]]} {incr ii} {
        if { $ii > 4 } { set where "fr2" }
  #puts "multiVal [set [join [list "::device:$deviceID" "(knobs)"] ""]] $ii"
        radiobutton $::mainf.frame$deviceID.$where.knob$ii -variable [subst {::v$deviceID,0}]  -value $ii -text [lindex [set [join [list "::device:$deviceID" "(items)"] ""]] $ii] -command [list selectDevice $deviceID [set [join [list "::device:$deviceID" "(knobs)"] ""]] $ii] -state $state
        pack $::mainf.frame$deviceID.$where.knob$ii -side left
      }
    }

  }


}

proc selectDevice { deviceID howManyTimes what } {
  set ::refreshDisabled yes
  set tmpValue [list $what ]
  for {set ii 1} {$ii < $howManyTimes} {incr ii} {
    lappend tmpValue $what
  }
  set finalValue [join $tmpValue ","]
  set ::device:${deviceID}(values) $finalValue
  eval exec "amixer -c $::currentCard -q cset iface=MIXER,numid=$deviceID [set ::device:${deviceID}(values)]"
  set ::refreshDisabled no
}

proc makeSliderWindow {card fillWindow} {
  if { [llength $::selectedControls($card)] } {
    set ::shownControls [expr {($::selected) ? $::selectedControls($card) : $::allControls}]
  } else {
    set ::shownControls $::allControls
  }
  # Construction of a new panel?
  if { $fillWindow } {
    set alreadyPacked [grid slaves $::mainf]
    if {[llength $alreadyPacked]} {
      foreach w [winfo children $::mainf] {
        destroy $w
      }
    }
    label $::mainf.infolabel -text [lindex $::cardInfo $card] -font "$::fontFamily -$::fontSize($::appSize) bold"
    grid $::mainf.infolabel -row 0 -columnspan 2
  }
  set jj 1
  set processedControls [list]
  foreach slider $::shownControls {
    if { [in $::allControls $slider] && [ni $processedControls $slider] } {
      set current_slider_no [set ::device:${slider}(numid)]
      placeKnobs $current_slider_no $jj $fillWindow
      incr jj
    }
    lappend processedControls $slider
  }
}

# Activate/deactivate "selected devices" menu button
proc selectedDevicesMB {} {
  if { [llength $::selectedControls($::currentCard)] } {
    $::imgSelected configure -palette ""
    .menuframe.selected configure -state normal
    return true
  } else {
    $::imgSelected configure -palette 32
    .menuframe.selected configure -state disabled
    return false
  }
}

proc displayedControls {} {
  set ::selected [expr {! $::selected}]
  if {$::selected} {
    .menuframe.selected configure -text "Selected"
  } else {
    .menuframe.selected configure -text "All"
  }
  makeSliderWindow $::currentCard yes
}

proc switchRefresh {} {
  if { [$::imgRefresh cget -palette] eq "32" } {
    $::imgRefresh configure -palette ""
    refreshWindow
  } else {
    $::imgRefresh configure -palette 32
    after cancel refreshWindow
  }
}

proc moveSlider { deviceID whichOne slValue } {
  set ::refreshDisabled yes
  if {$::together} {
    set otherControls [list]
    for {set ii 0} {$ii < [set ::device:${deviceID}(knobs)]} {incr ii} {
      lappend otherControls $ii
    }
    set lastval [split [set ::device:${deviceID}(values)] ","]
#    set otherControls [lreplace $otherControls $whichOne $whichOne]
    set otherControls [lreplace [K $otherControls [set otherControls {}]] $whichOne $whichOne]
    foreach number $otherControls {
      incr [subst {::v$deviceID,$number}] [expr {$slValue - [lindex $lastval $whichOne]}]
      set lastval [lreplace $lastval $number $number [set [subst {::v$deviceID,$number}]]]
    }
#    set lastval [lreplace $lastval $whichOne $whichOne $slValue]
    set lastval [lreplace [K $lastval [set lastval {}]] $whichOne $whichOne $slValue]
    set ::device:${deviceID}(values) [join $lastval ","]
  } else {
    set lastval [split [set ::device:${deviceID}(values)] ","]
#    set lastval [lreplace $lastval $whichOne $whichOne $slValue]
    set lastval [lreplace [K $lastval [set lastval {}]] $whichOne $whichOne $slValue]
    set ::device:${deviceID}(values) [join $lastval ","]
  }
  if [set [join [list "::device:$deviceID" "(operational)"] ""]] {
    eval exec "amixer -c $::currentCard -q cset iface=MIXER,numid=$deviceID [set ::device:${deviceID}(values)]"
  }
  set ::refreshDisabled no
}

proc pressButton { deviceID whichOne } {
  set ::refreshDisabled yes
  if {$::together} {
    set lastval [list]
    for {set ii 0} {$ii < [set ::device:${deviceID}(knobs)]} {incr ii} {
      lappend lastval [set [subst {::v$deviceID,$whichOne}]]
      set [subst {::v$deviceID,$ii}] [set [subst {::v$deviceID,$whichOne}]]
    }
    set ::device:${deviceID}(values) [join $lastval ","]
  } else {
    set lastval [split [set ::device:${deviceID}(values)] ","]
    set lastval [lreplace $lastval $whichOne $whichOne [set [subst {::v$deviceID,$whichOne}]]]
    set ::device:${deviceID}(values) [join $lastval ","]
  }
  eval exec "amixer -c $::currentCard -q cset iface=MIXER,numid=$deviceID [set ::device:${deviceID}(values)]"
  set ::refreshDisabled no
}

# Po wywoaniu checkSettings zawsze naley jeszcze wywoa makeSliderWindow
proc checkSettings {card} {
  set ::allControls [list]
  set ::allValues [list]

  if { [catch {exec amixer contents -c $card} response] } {
    puts "[::msgcat::mc noAmixer $response]"
    exit 1
  }

  if { ![string equal -length 6 "numid=" $response] } {
    puts "[::msgcat::mc unknownResponse $response]"
    exit 1
  }

  set current_position [string last "numid=" $response end]
  set end_of_paragraph [string length $response]
  set txtcontrols [list]
  while {$current_position >= 0} {
    lappend txtcontrols [string range $response $current_position $end_of_paragraph]
    set end_of_paragraph [expr {$current_position - 2}]
    set current_position [string last "numid=" $response $current_position]
  }

  set whole_line ""
  foreach paragraph $txtcontrols {
    array set tmpdesc { numid "" iface "" name "" type "" access "" knobs ""
      min "" max "" step "" values "" dBscalemin "" dBstep "" mute "" items {}
      operational no}

    regexp {numid=(\d+),iface=(\w+),name='(.+)'\n  ; type} $paragraph whole_line tmpdesc(numid) tmpdesc(iface) tmpdesc(name)
    set optional_rest ""
    if {[string first "ENUMERATED" $paragraph] == -1} {
      regexp {;\stype=(INTEGER|BOOLEAN|IEC958),access=(.{5,8}),values=(\d)(,min=(\d+),max=(\d+),step=(\w+)|)} $paragraph whole_line tmpdesc(type) tmpdesc(access) tmpdesc(knobs) optional_rest minval maxval stepval
      if {[string length $optional_rest]} {
        set tmpdesc(min)  $minval
        set tmpdesc(max)  $maxval
        set tmpdesc(step) $stepval
      }
    } else {
      regexp {;\stype=(ENUMERATED),access=(.{5,8}),values=(\d),items=(\d+)} $paragraph whole_line tmpdesc(type) tmpdesc(access) tmpdesc(knobs) itemCounter
      if {$itemCounter} {
        for {set itemCounter} {$itemCounter > 0} {incr itemCounter -1} {
#          regexp {;\sItem #\d '(\w+)'} $paragraph whole_line urzadzenie
          regexp {;\sItem #\d '([0-9a-zA-Z\s]+)'} $paragraph whole_line urzadzenie
          lappend tmpdesc(items) $urzadzenie
          set paragraph [string range $paragraph [expr {[string first "Item #" $paragraph] +1}] end]
        }
      }
    }
    regexp {:\svalues=([\w,]+)\s*} $paragraph whole_line tmpdesc(values)
    set tmpdesc(dBscalemin)	""
    set tmpdesc(dBstep)		""
    set tmpdesc(mute)		""
    set whole_line		""
    if {$tmpdesc(iface) eq "MIXER"} {
      regexp {\s\|\sdBscale-min=(.+),step=(.+),mute=(\d)} $paragraph whole_line dBscalemin dBstep mute
      if {[string length $whole_line]} {
        set tmpdesc(dBscalemin)	$dBscalemin
        set tmpdesc(dBstep)	$dBstep
        set tmpdesc(mute)	$mute
      }
      array set "::device:$tmpdesc(numid)" [array get tmpdesc]
      lappend ::allControls $tmpdesc(numid)
      lappend ::allValues $tmpdesc(values)
    }
  }
}

proc glueSliders {} {
  set ::together [expr {! $::together}]
  if {$::together} {
#    .menuframe.together configure -text "Tog"
    $::imgChain configure -palette ""
  } else {
#    .menuframe.together configure -text "Sep"
    $::imgChain configure -palette 32
  }
}

proc K { x y } { return $x }

proc switchCard { whichOne } {
  set ::refreshDisabled yes
  if {$whichOne eq "lastUsed"} {
    if { $::currentCard > [lindex $::cards end] } {
      set ::currentCard 0
    }
  } else {
    set ::currentCard [expr {($::currentCard == [lindex $::cards end]) ? 0 : [lindex $::cards [expr {$::currentCard + 1}]] }]
  }
#  .menuframe.card configure -text $::currentCard
  checkSettings $::currentCard
  selectedDevicesMB
#  set shownControls [expr {($::selected) ? $::selectedControls($::currentCard) : $::allControls}]
  makeSliderWindow $::currentCard yes
  detectSets $::currentCard
  set ::refreshDisabled no
}

proc setPreferences {} {
  if {[winfo exists .sp]} {
    tk_messageBox -icon error -type ok -message "[::msgcat::mc prefErrWinOpen]"
    return
  }
  toplevel .sp
  wm title .sp "[::msgcat::mc prefTitle]"
  label .sp.remark -padx 5 -pady 5 -text "[::msgcat::mc prefWarning]"
  grid .sp.remark -row 0 -column 0 -columnspan 2
  label .sp.size -text "[::msgcat::mc prefSize]"
  grid .sp.size -row 1 -column 0
  frame .sp.frsize
  radiobutton .sp.frsize.small -text "[::msgcat::mc prefSmall]" -variable ::appSize -value "small"
  radiobutton .sp.frsize.medium -text "[::msgcat::mc prefMedium]" -variable ::appSize -value "medium"
  radiobutton .sp.frsize.big -text "[::msgcat::mc prefBig]" -variable ::appSize -value "big"
  grid .sp.frsize -row 1 -column 1
  pack .sp.frsize.small .sp.frsize.medium .sp.frsize.big -side left
  frame .sp.frcolour
  grid .sp.frcolour -row 2 -column 0
  label .sp.frcolour.forspin -text "[::msgcat::mc prefColourTheme]"
  set tmpColourTheme [expr {([string length $::colourTheme]) ? $::colourTheme : "gray85"}]
  spinbox .sp.frcolour.scheme -width 12 -textvariable ::colourTheme -values $::bgColours -state normal -command { .sp.colour configure -bg $colourTheme ; .sp.colour.demo configure -bg $colourTheme -foreground [lindex $::fgColours [lsearch $::bgColours $colourTheme]] }
  set ::colourTheme $tmpColourTheme
  unset tmpColourTheme
  pack .sp.frcolour.forspin .sp.frcolour.scheme -side left
  frame .sp.colour
  grid .sp.colour -row 2 -column 1
  label .sp.colour.demo -relief raised -padx 5 -pady 5 -text "[::msgcat::mc prefExample]"
  pack .sp.colour.demo -pady 10 -padx 10
  for {set ii 0} {$ii < [llength $::cards]} {incr ii} {
    label .sp.card$ii -text "[::msgcat::mc prefChosenDevs $ii]"
    grid .sp.card$ii -row [expr {3 + $ii}] -column 0
    entry .sp.entry$ii -width 20 -textvariable ::selectedControls($ii)
    grid .sp.entry$ii -row [expr {3 + $ii}] -column 1
  }
  checkbutton .sp.loadset -text "[::msgcat::mc prefLoadAlways]" \
      -offvalue no -onvalue yes -variable ::loadAlways
  grid .sp.loadset -row [expr {3 + $ii}] -column 0 -columnspan 2
  checkbutton .sp.saveset -text "[::msgcat::mc prefSaveAlways]" \
      -offvalue no -onvalue yes -variable ::saveAlways
  grid .sp.saveset -row [expr {4 + $ii}] -column 0 -columnspan 2
  checkbutton .sp.savewin -text "[::msgcat::mc prefSaveWindow]" \
      -offvalue no -onvalue yes -variable ::saveWindow
  grid .sp.savewin -row [expr {5 + $ii}] -column 0 -columnspan 2
  frame .sp.buttons
  grid .sp.buttons -row [expr {6 + $ii}] -column 0 -columnspan 2
  button .sp.buttons.save -text "[::msgcat::mc prefSave]" -command { selectedDevicesMB ; saveSettings ; destroy .sp }
  button .sp.buttons.exit -text "[::msgcat::mc prefExit]" -command { selectedDevicesMB ; destroy .sp }
  button .sp.buttons.help -text "[::msgcat::mc prefHelp]" -command { explain "[::msgcat::mc prefHelpTitle]" "[::msgcat::mc prefHelpMsg]" }
  pack .sp.buttons.save .sp.buttons.exit .sp.buttons.help -side left
}

proc explain { title txtMessage } {
  toplevel .msgwin
  wm title .msgwin $title
  message .msgwin.msg -padx 5 -pady 5 -text $txtMessage
  button .msgwin.b -text OK -command { destroy .msgwin }
  pack .msgwin.msg .msgwin.b
  grab .msgwin
}

proc saveSettings {} {
  set names [list "::appSize" "::colourTheme" "::loadAlways" "::saveAlways" "::saveWindow" "::currentCard" "::selected"]
  set fileName [file join $::env(HOME) ".config" "tkamixer" "config"]
  if { ! [file exists [file dirname $fileName]] } {
    file mkdir [file dirname $fileName]
  } elseif { [file isdirectory $fileName] } {
    tk_messageBox -icon error -type ok -message "[::msgcat::mc prefErrDirPresent $fileName]"
    return
  } elseif { [file isdirectory [file dirname $fileName]] && [expr {! [file writable [file dirname $fileName]]}] } {
    tk_messageBox -icon error -type ok -message "[::msgcat::mc prefErrNoWrite [file dirname $fileName]]"
    return
  } elseif { [file isfile $fileName] && [expr {! [file writable $fileName]}] } {
    tk_messageBox -icon error -type ok -message "[::msgcat::mc prefErrNoWrite $fileName]"
    return
  }

  set fd [open $fileName w]
  for {set ii 0} {$ii < [llength $names]} {incr ii} {
    puts $fd "set [lindex $names $ii] [set [lindex $names $ii]]"
  }
  if $::saveWindow { puts $fd "set ::mainWinGeometry [wm geometry .]" }

  if $::saveAlways {
    for {set ii 0} {$ii < [llength $::cards]} {incr ii} {
      puts $fd "\nlappend mixers \{[lindex $::cardInfo $ii]\}"
      if {[llength $::selectedControls($ii)]} {
        puts $fd "set selectedControls_$ii [list $::selectedControls($ii)]"
      }
      checkSettings $ii
      puts $fd "set ::devices_$ii.0 \{$::allControls\}"
      puts $fd "set ::values_$ii.0 \{$::allValues\}"
      for {set jj 1} {$jj < 10} {incr jj} {
        if {[info exists [subst {::values_$ii.$jj}]] && [llength [set [subst {::values_$ii.$jj}]]]} {
          puts $fd "set ::devices_$ii.$jj \{[set ::devices_$ii.$jj]\}"
          puts $fd "set ::values_$ii.$jj \{[set ::values_$ii.$jj]\}"
        }
      }
    }
    if {[expr {$ii - 1}] != $::currentCard} {checkSettings $::currentCard}
  }

  if { $::pulseAudio && $::pa(state) eq "normal" } {
    puts $fd "\nif \{ \$::pulseAudio \&\& \$::pa(state) eq \"normal\" \} \{"
    puts $fd "  array set ::pa \[list [array get ::pa]\]"
    puts $fd "\}"
  }

  if {[catch {close $fd} err]} {
    tk_messageBox -icon error -type ok -message $err
  }
}

proc loadSettings {} {
  set fileName [file join $::env(HOME) ".config" "tkamixer" "config"]
  if { [file isdirectory $fileName] } {
    tk_messageBox -icon error -type ok -message "[::msgcat::mc prefErrDirPresent]"
    return
  } elseif {[file isfile $fileName] && [expr {! [file readable $fileName]}]} {
    tk_messageBox -icon error -type ok -message "[::msgcat::mc prefErrNoRead]"
    return
  } elseif {! [file exists $fileName]} {
    return
  }
  set mixers [list]
  set fd [open $fileName r]
  set configScript [read $fd]
  close $fd
  eval $configScript
  # Usun wszystkie zapisane zbiory ustawie (take 0=domylny), jeli
  # wykryto inne karty, ni byy uprzednio
  for {set ii 0} {$ii < [llength $mixers]} {incr ii} {
    set gotIt [lsearch -exact $::cardInfo [lindex $mixers $ii]]
    if { $gotIt != $ii } {
      for {set jj 0} {$jj < 10} {incr jj} {
        set ::devices_[set ii].$jj [list]
        set ::values_[set ii].$jj [list]
      }
    }
  }

  if {$::loadAlways || ($::argc > 0 && $::argv eq "init")} {
    if {[llength $mixers]} {
      for {set ii 0} {$ii < [llength $mixers]} {incr ii} {
        set gotIt [lsearch -exact $::cardInfo [lindex $mixers $ii]]
        if { $gotIt == $ii } {
          if {[info exists [subst {selectedControls_$ii}]]} {
            set ::selectedControls($gotIt) [set [subst {selectedControls_$ii}]]
          }
        # Below: 'catch' for handle disabled devices with no complaining
          for {set jj 0} {$jj < [llength [set ::devices_$ii.0]]} {incr jj} {
            catch { eval exec "amixer -c $gotIt -q cset numid=[lindex [set ::devices_$ii.0] $jj] [lindex [set ::values_$ii.0] $jj]" }
          }
        } else {
          for {set jj 0} {$jj < 10} {incr jj} {eraseSet $jj}
        }
      }
    }
    if {$::argc > 0 && $::argv eq "init"} {
      exit 0
    }
  }
}

proc refreshWindow {} {
  if { ! $::refreshDisabled } {
    checkSettings $::currentCard
    makeSliderWindow $::currentCard no
    if { $::pulseAudio && $::pa(state) eq "normal" } { getCurrentPA }
  }
  after 500 refreshWindow
}

# Ponisze procedury dotycz obsugi zbiorw ustawie dwiku (ten rzd 9-ciu przyciskw)
proc detectSets { card } {
  for {set jj 1} {$jj < 10} {incr jj} {
    if { [info exists [subst {::values_$card.$jj}]] && [llength [set [subst {::values_$card.$jj}]]] } {
      .setsframe.$jj.load   configure -state normal
      .setsframe.$jj.delete configure -state normal
    } else {
      .setsframe.$jj.load   configure -state disabled
      .setsframe.$jj.delete configure -state disabled
    }
  }
}

proc eraseSet { whichButton } {
global currentCard
  set ::devices_[set currentCard].$whichButton [list]
  set ::values_[set currentCard].$whichButton [list]
  .setsframe.$whichButton.load configure   -state disabled
  .setsframe.$whichButton.delete configure -state disabled
}

proc recordSet { whichButton } {
global currentCard
  checkSettings $currentCard
  makeSliderWindow $currentCard no
  set ::devices_[set currentCard].$whichButton $::allControls
  set ::values_[set currentCard].$whichButton $::allValues
  .setsframe.$whichButton.load configure   -state normal
  .setsframe.$whichButton.delete configure -state normal
}

proc activateSet { whichButton } {
global currentCard
# 'catch' for handle disabled devices with no complaining
  for {set ii 0} {$ii < [llength [set ::values_$currentCard.$whichButton]]} {incr ii} {
    catch { eval exec "amixer -c $currentCard -q cset numid=[lindex [set ::devices_$currentCard.$whichButton] $ii] [lindex [set ::values_$currentCard.$whichButton] $ii]" }
  }
  checkSettings $currentCard
  makeSliderWindow $currentCard no
}

# ----------------- variable settings -----------------------
array set fontSize    {small 12 medium 16 big 20}
array set scaleLength {small 300 medium 400 big 500}
array set ySbarWidth  {small 10 medium 12 big 15}
array set sliderWidth {small 10 medium 12 big 15}
array set padxy       {small 0 medium 2 big 10}

set bgColours [list gray85 bisque bisque3 bisque4 SteelBlue3 SteelBlue4 turquoise3 \
  turquoise4 SpringGreen4 OliveDrab4 khaki4 gold3 goldenrod2 sienna3 salmon4 \
  orange3 orange4 HotPink1 HotPink4 magenta4 purple4 black]
set fgColours [list Black Brown4 Black Yellow GreenYellow White Black Yellow wheat1 White \
  White Black Black White bisque MintCream gold LemonChiffon White White AliceBlue LightBlue]
set slColours [list maroon chocolate3 DarkSeaGreen3 DarkOrchid1 pink DarkOrange gold blue2 coral DarkMagenta \
  firebrick DimGrey DeepPink Black LightSkyBlue3 red3 DarkViolet LightSkyBlue LightPink DodgerBlue orange burlywood2]

set pulseAudio no
set selected no
set together true
set colourTheme ""
set loadAlways no
set saveAlways no
set saveWindow no
set appSize medium
set mainWinGeometry ""

# http://wiki.tcl.tk/1710
set scriptPath   $::argv0
set workingPath  [file dirname $::argv0]
while { ![catch {file readlink $scriptPath} result] } {
   cd $workingPath
   set scriptPath  [file join [pwd] $result]
}

package require msgcat
set currentLanguage [string range [::msgcat::mclocale] 0 [expr [string first "_" [::msgcat::mclocale]] -1]]
if { [::msgcat::mclocale] eq "c" } {
  ::msgcat::mclocale en
} elseif { ! [file readable [file join [file dirname $scriptPath] $currentLanguage.msg]] } {
  ::msgcat::mclocale en
}
::msgcat::mcload [file dirname $scriptPath]

if { $currentLanguage eq "pl" } {
  set readmeFile [file join [file dirname $scriptPath] "PRZECZYTAJ.TO"]
} else {
  set readmeFile [file join [file dirname $scriptPath] "README.TXT"]
}

# ------------------------ main --------------------------------------
if { [catch {exec amixer info} response] } {
  puts "[::msgcat::mc noAmixer $response]"
  exit 1
}

set refreshDisabled yes
set currentCard 0
set cards [list]
set cardInfo [list]
while { ! [catch {exec amixer info -c $currentCard} response] } {
  lappend cards $currentCard
  set response [string range $response 0 [expr {[string first "\n" $response] - 1}]]
  set a [expr {[string first "\'\/\'" $response] + 3}]
  set b [expr {[string first " at " $response] - 1}]
  if {$b < $a} {set b [expr {[string last "'" $response] - 1}]}
  lappend cardInfo [string range $response $a $b]
  incr currentCard
}
unset a
unset b
incr currentCard -1

if {$currentCard == -1} {
  tk_messageBox -message "[::msgcat::mc noSoundcard]" -icon error
  exit 1
} else {
  set currentCard 0
}

# Handle PulseAudio
proc getCurrentPA {} {
  catch {exec pactl -- get-sink-mute @DEFAULT_SINK@} response
  regexp -all {.+:\s+(\w+)} $response aaa what
  set ::pa(mute) [expr {[string index $what 0] eq "n" ? "false" : "true"}]
  catch {exec pactl -- get-sink-volume @DEFAULT_SINK@} response
  regexp -all {.+\/\s+(\d+)%\s.+\s+\/\s+(\d+)%\s.+} $response aaa ::pa(volLeft) ::pa(volRight)
  set ::pa(fvl) $::pa(volLeft)
  set ::pa(fvr) $::pa(volRight)
}

set response ""
if { [catch {exec amixer contents} response] } {
  puts "[::msgcat::mc noAmixer $response]"
  set response ""
}
if { [string first "MIXER" $response] != -1 } {
  array set ::pa {tmp "" mute "false" state disabled fvl 100 fvr 100 volLeft 100 volRight 100 volLimit 100}
  set ::rbCommand "" ; set ::scCommandL "" ; set ::scCommandR ""
  set ::pa(state) [expr {[auto_execok pactl] eq "" ? "disabled" : "normal"}]

  set response ""
  catch {exec amixer info} response
  if { [string first "PulseAudio" $response] != -1 } {
    set ::pulseAudio yes
  } else {
    set ::pulseAudio no
    set ::pa(state) "disabled"
  }

  if { $::pa(state) eq "normal" } {
    getCurrentPA
    set ::pa(volLimit) 100
    if { $::pa(volLeft) > $::pa(volLimit) || $::pa(volRight) > $::pa(volLimit) } {
      incr ::pa(volLimit) 100
    }
    if { $::pa(volLeft) > $::pa(volLimit) || $::pa(volRight) > $::pa(volLimit) } {
      incr ::pa(volLimit) 100
    }
    set ::scCommandL {if $::together { incr ::pa(volRight) [expr {$::pa(volLeft)-$::pa(fvl)}] ; set ::pa(fvr) $::pa(volRight) } ; eval exec pactl -- set-sink-volume @DEFAULT_SINK@ [set ::pa(volLeft)]% [set ::pa(volRight)]% ; set ::pa(fvl) }
    set ::scCommandR {if $::together { incr ::pa(volLeft) [expr {$::pa(volRight)-$::pa(fvr)}] ; set ::pa(fvl) $::pa(volLeft) } ; eval exec pactl -- set-sink-volume @DEFAULT_SINK@ [set ::pa(volLeft)]% [set ::pa(volRight)]% ; set ::pa(fvr) }
    set ::rbCommand {
      set ::pa(tmp) $::together
      set ::together no
      if { $::pa(volLeft)  > $::pa(volLimit) } { set ::pa(volLeft)  [set ::pa(fvl) $::pa(volLimit)] }
      if { $::pa(volRight) > $::pa(volLimit) } { set ::pa(volRight) [set ::pa(fvr) $::pa(volLimit)] }
      .default.volr configure -tickinterval [expr {$::pa(volLimit)/10}] -to $::pa(volLimit)
      .default.voll configure -to $::pa(volLimit)
      .default.volr configure -to $::pa(volLimit)
      eval exec pactl -- set-sink-volume @DEFAULT_SINK@ [set ::pa(volLeft)]% [set ::pa(volRight)]%
      set ::together $::pa(tmp)
    }
  }
}


# using a module "scrolled frame" http://wiki.tcl.tk/9223
source [file join [file dirname $scriptPath] sf9223.2.tcl]

namespace import ::scrolledframe::scrolledframe

set allControls [list]
for {set cb 0} {$cb < [llength $cards]} {incr cb} {
  array set selectedControls [list $cb [list]]
}

loadSettings
if { $::currentCard > [lindex $::cards end] } { set ::currentCard 0 }

if {$argc > 0 && [in [list small medium big] [lindex $argv 0]] } {
  set appSize [lindex $argv 0]
}
#puts $appSize
package require Tk
wm title . "Tkamixer"

if {$::colourTheme ne "" && $::colourTheme ne "gray85"} {
  catch {tk_setPalette background $::colourTheme foreground [lindex $::fgColours [lsearch $::bgColours $colourTheme]] selectColor [lindex $::slColours [lsearch $::bgColours $colourTheme]]} err
}


checkSettings $::currentCard
set fontFamily [firstFont {Verdana Arial} Helvetica]
#set fontFamily [firstFont {Terminus Courier} Fixed]
option add *Label.font "$fontFamily -$::fontSize($::appSize)" widgetDefault
option add *Message.font "$fontFamily -$::fontSize($::appSize)" widgetDefault
option add *Scale.font "$fontFamily -$::fontSize($::appSize)" widgetDefault
option add *Scale.borderWidth 1 widgetDefault
if {$appSize eq "small"} {
  option add *Scale.sliderLength 20 widgetDefault
}
option add *Checkbutton.font "$fontFamily -$::fontSize($::appSize)" widgetDefault
###option add *Checkbutton.selectColor orange widgetDefault ;# default: maroon
#option add *Checkbutton.borderWidth 1 widgetDefault
#option add *Label.borderWidth 1 widgetDefault
#option add *Spinbox.borderWidth 1 widgetDefault
option add *Spinbox.font "$fontFamily -$::fontSize($::appSize)" widgetDefault
#option add *Entry.borderWidth 1 widgetDefault
option add *Button.font "$fontFamily -$::fontSize($::appSize) bold" widgetDefault
#option add *Button.borderWidth 1 widgetDefault
option add *Radiobutton.font "$fontFamily -$::fontSize($::appSize)" widgetDefault
###option add *Radiobutton.selectColor orange widgetDefault ;# default: maroon
#option add *Radiobutton.borderWidth 1 widgetDefault
# option add *font {{Times New Roman} -14 bold italic}

# Icons for buttons were taken from Anthony's Icon Archive (and a little
# bit changed/simplified) - http://www.cit.gu.edu.au/images/Images.html

set imgCard [image create photo -data {
R0lGODlhIAAgAMIGAAAAAACAAACAgICAAICAgP//gP///////yH5BAEAAAcALAAAAAAgACAA
AAPmeLrc/jDKSasi2FoCuq7ctwELZp5oqoYHUbxwLM802RZCjufC3vM/nJBnc/F8P93RwGwa
dsVdEyCYJgUBA4qaA3CMVQxVy7kWyGIA9AvtUN1UX9bkJrKVRyAuEDhz4Gtee1ZWe3wGcB12
glhkY2I9L32JHQRddwFwmW59PXxDlgNYmZiGnXydBZ9BnnxejFlMALGIqXNljq5sh2K3s5YE
m4qawi6unJoDAcpyra4XqqjRfMrL1tOfqJkLlN3e3h8DBOIFLuYFRTYW4uTnLunh4/Ls4/Aa
324X6iISLPz9+/4JHEiQQgIAOw==}]

set imgPrefs [image create photo -data {
R0lGODlhIAAgAMIDAAAAAAAA/6pVVf///////////////////yH5BAEKAAQALAAAAAAgACAA
AAOXSLrc/jDK2YBdFuCcKQGDpoChNZxoSIHil6JBjLYPqbpvHLx0NY+nDM/WY7A2LY6IVMTc
IEdnEzilVqpR6NOTrW1XX1/VO+6Cy2FJJ7LmCt5TwFswxszhDvm9/qH7i3KBdCt/gxWFfIKB
eYiEioyPE4qGRo2SlpWRaph2mmycI6A1opOJpKKMe6lzfH14o69cF2yzHrYUCQA7}]

set imgSelected [image create photo -data {
R0lGODlhIAAgAMIDAAAAAP+qAP//qv///////////////////yH5BAEKAAQALAAAAAAgACAA
AAPDSLrc/jASQKV1dIhRr6RbqHUek4kCBWzkBWrwmwIjYJ3wnMcrZ2M9kWwXOpFwqBWSFwj6
JjMUR0CVUQJY2qwJrWpH1LAKSx5nu+LeKlwll93cNXsmF8ObcFvdvk/dAXlobIAgg3mAbnpz
bVlyiE2PiYJ2iUqQfn+TjJc+jndxi35LLJaZfVFEOVemoV6pq5+KrV5FkbGaoWafkoWzm7uN
C72ttn8/DcNzxWXHEMm/zCUTe7Yt0r2P1tLCltrbyN7f4g0JADs=}]

set imgChain [image create photo -data {
R0lGODlhIAAgAMIEAAAAAFVVVVWqqgD//////////////////yH5BAEKAAcALAAAAAAgACAA
AAO2eAFw/mfASWcQYlFZO7xE1kCcZ4Iiaa6KEGpOyXbgAMvy7NQLYDM53c71G2AGI6GFGDAO
QknlrjawuQRRU3XLrTa7YF3gtR1CpYqQTcJBZVmBZobTvr5XRvUDyXDdPRI1I009BH8rF0YN
FyIMaDtPWCg9jzF2k4csVi+YlYEZcZ1oQS0vDKGZezRXPRhYmh6YAHZ4sZgotauTpbkVY3EY
AUO9FGOSwg/IgLG0SqTJzZXF0dIf0QkAOw==}]

set imgQuestion [image create photo -data {
R0lGODlhIAAgAMIEAKoAAP8AAFWqqv+qAP///////////////yH5BAEKAAQALAAAAAAgACAA
AAObSLrc/tCNSeuIeIbNO8eSB4wkuQmgMnRlWwZoxpqcC8DRWpNCT2w2QKyhA/J8CmMLySgu
hwSdi7lwLpvK42NXYkqfkOwIae1CG9le7Gs+owNjslhIhajl8Gfdjp+qUyF5R3+AWIJxe4Bs
iG6Fi3eFYYKQkZJ0iZULd42ZUUacmStfnSGiRqSGVqCFHnmYkTOEqEmDq5GbtpWUIAkAOw==}]

set imgExit [image create photo -data {
R0lGODlhIAAgAOMLAAAAAFVVAP8AAKpVAP9VAKqqAKqqqv+qAP+qVf+qqv//qv//////////
/////////yH5BAEAAA8ALAAAAAAgACAAAAT+8MlJ37o462uql8YCjGRpAkb3USGgvHCcAEWB
qisoxvxcB7fcpMWD+QoBYEr4GO2Kx6QStwIcRlAaUhoUWq89rTTZzX3Br8RsMBhPmWewet0e
l5unM4EAUBsAbG5lcQeFhXuIN4B1XEsShIeIkiiLgo5NhpmSk3+BdpeEm5ydjGSXeCebN6RA
AKZUDymyf6Mpiy6ubxWztHurtmy4r3C+vKwjw14CI8aVn7AezL/ApbpCts2ejdAeLXnaydci
hQgIL87bTLEi5TAI6Ml5J+7m8K0k5i/t+vkK+wrwvvwb2I8gOisE6Snkd84TgIQMI/orWOnh
Qn0XE95aWGFixn4FJCJSiAAAOw==}]

set imgRefresh [image create photo -data {
R0lGODlhIAAgAOMMAABtAACSACSSJEmSSSS2JEm2JEm2SW22bW3bSZK2tpLbtrbb2///////
/////////yH5BAEKAA8ALAAAAAAgACAAAAS48MlJq704680FAAEnDl9ZDhYgPp7potMnlkht
I0Yp6Rt9/5+WLHNwmQQ2g6Hg0hiNOKVBIGCqMqeKULosWLEARaYl6HqHFsXV+fG60ZV1D+Am
EOArSqBdsN/leTFPeHlqg4Q7h2GBFIqMcQBAiHkfkiuAgpKYKWIXlTdkGYZgSVIlAhUkk4kI
VFw5ipuCp1JeQkYHbC5uXn52JZ1gKgsldX5CwWB6iiEzuRSqRjAiqBcJJgmP2tvbEQA7}]
$::imgRefresh configure -palette 32 ; # Default B/W

set imgErase [image create photo -data {
R0lGODlhBwAHAIABAP8AAP///yH5BAEKAAEALAAAAAAHAAcAAAINDIIHa7yP2klPORdPAQA7}]
set imgRecord [image create photo -data {
R0lGODlhBwAHAIABAP8AAP///yH5BAEKAAEALAAAAAAHAAcAAAIMjG+gmgu9IAvuoVMAADs=}]


frame .menuframe
button .menuframe.card -image $imgCard -relief flat -overrelief raised -command {switchCard next}
button .menuframe.together -image $imgChain -relief flat -overrelief raised -command glueSliders
button .menuframe.selected -image $imgSelected -relief flat -overrelief raised -command displayedControls
button .menuframe.refresh -image $imgRefresh -relief flat -overrelief raised -command switchRefresh
button .menuframe.preferences -image $imgPrefs -relief flat -overrelief raised -command setPreferences
button .menuframe.about -image $imgQuestion -relief flat -overrelief raised -command { appHelp $readmeFile } ;#{ explain "[::msgcat::mc aboutTitle]" "[::msgcat::mc aboutMsg]" }
button .menuframe.exit -image $imgExit -relief flat -overrelief raised -command { saveSettings ; exit 0 }
pack .menuframe
balloon .menuframe.card "[::msgcat::mc buttonSwitchCard]"
balloon .menuframe.together "[::msgcat::mc buttonGlue]"
balloon .menuframe.selected "[::msgcat::mc buttonVisibleSliders]"
balloon .menuframe.refresh "[::msgcat::mc buttonRefresh]"
balloon .menuframe.preferences "[::msgcat::mc buttonPreferences]"
balloon .menuframe.about "[::msgcat::mc buttonAbout]"
balloon .menuframe.exit "[::msgcat::mc buttonExit]"
pack .menuframe.card .menuframe.selected .menuframe.together .menuframe.refresh .menuframe.preferences -side left -padx $padxy($appSize) -pady $padxy($appSize)
# if { $appSize ne "small" } {
  pack .menuframe.about .menuframe.exit -side left -padx $padxy($appSize) -pady $padxy($appSize)
# }

frame .setsframe
if { $appSize eq "small" } { set tmpPad 0 ; set tmpIpad 0
} elseif { $appSize eq "medium" } { set tmpPad 0 ; set tmpIpad 1
} else { set tmpPad 1 ; set tmpIpad 1
}
for { set ii 1 } { $ii < 10 } { incr ii } {
  frame .setsframe.$ii
  button .setsframe.$ii.load   -text $ii -padx $tmpPad -pady $tmpPad -highlightthickness $tmpPad -state disabled -command "activateSet $ii"
  button .setsframe.$ii.delete -image $imgErase  -padx $tmpPad -pady $tmpPad -highlightthickness $tmpPad -state disabled -command "eraseSet $ii"
  button .setsframe.$ii.record -image $imgRecord -padx $tmpPad -pady $tmpPad -highlightthickness $tmpPad -command "recordSet $ii"
  pack .setsframe.$ii.load   -side top   -expand yes -fill both -ipadx $tmpIpad -ipady $tmpIpad -padx $tmpPad -pady $tmpPad
  pack .setsframe.$ii.delete -side left  -expand yes -fill both -ipadx $tmpIpad -ipady $tmpIpad -padx $tmpPad -pady $tmpPad
  pack .setsframe.$ii.record -side right -expand yes -fill both -ipadx $tmpIpad -ipady $tmpIpad -padx $tmpPad -pady $tmpPad
  pack .setsframe.$ii -side left -padx [expr {$tmpPad + $tmpIpad}] -pady [expr {$tmpPad + $tmpIpad}]
  bind . <Key-$ii> ".setsframe.$ii.load flash ; .setsframe.$ii.load invoke"
}
#.setsframe.1.load configure -state normal
pack .setsframe

if {[llength $cards] == 1} {
  .menuframe.card configure -state disabled
}
selectedDevicesMB

# Pulseaudio as the default mixer device in its dedicated labelframe
if { $pulseAudio } {
  labelframe .default -text "PulseAudio"
  pack .default -fill x  -ipadx $tmpIpad -ipady $tmpIpad -padx $tmpPad -pady $tmpPad
  checkbutton .default.mute -variable ::pa(mute) -offvalue false -onvalue true -text "[::msgcat::mc mute]" -command {eval exec pactl -- set-sink-mute @DEFAULT_SINK@ $::pa(mute)} -state $::pa(state)
  pack .default.mute
  scale .default.voll -from 0 -to $::pa(volLimit) -length $::scaleLength($::appSize) -width $::sliderWidth($::appSize) \
      -orient horizontal -variable ::pa(volLeft) -showvalue false -state $::pa(state) -command $::scCommandL \
      -resolution 5 -label "[::msgcat::mc left]"
  pack .default.voll -expand 1 -fill x
  scale .default.volr -from 0 -to $::pa(volLimit) -length $::scaleLength($::appSize) -width $::sliderWidth($::appSize) \
      -orient horizontal -variable ::pa(volRight) -showvalue false -state $::pa(state) -command $::scCommandR \
      -resolution 5 -tickinterval [expr {$::pa(volLimit)/10}] -label "[::msgcat::mc right]"
  pack .default.volr -expand 1 -fill x
  label .default.radiogroup -text "[::msgcat::mc volLimit]"
  pack .default.radiogroup -side left -fill x -expand yes
  radiobutton .default.max1 -variable ::pa(volLimit) -value 100 -text "100%" \
      -state $::pa(state) -command $::rbCommand
  pack .default.max1 -side left
  radiobutton .default.max2 -variable ::pa(volLimit) -value 200 -text "200%" \
      -state $::pa(state) -command $::rbCommand
  pack .default.max2 -side left
  radiobutton .default.max3 -variable ::pa(volLimit) -value 300 -text "300%" \
      -state $::pa(state) -command $::rbCommand
  pack .default.max3 -side left
}
unset tmpPad
unset tmpIpad
# Sterowanie pulseuadio przy pomocy pactl
# (jeli dla kadego kanau osobno, naley podadwie wartoci jako parametry,
# dla lewego i prawego kanau, np. "... 100% 200%")
#pactl set-sink-mute @DEFAULT_SINK@ true
#pactl set-sink-mute @DEFAULT_SINK@ false
#pactl -- set-sink-volume @DEFAULT_SINK@ 200%
#pactl -- set-sink-volume @DEFAULT_SINK@ +10%
#pactl -- set-sink-volume @DEFAULT_SINK@ -10%
#pactl -- get-sink-volume @DEFAULT_SINK@


scrollbar .sby -command ".sf yview" -width $::ySbarWidth($::appSize)
scrolledframe .sf -fill x -yscrollcommand ".sby set" -height [expr {$::scaleLength($::appSize) * 1.5}] -width [expr {$::scaleLength($::appSize) + 30}]
set mainf .sf.scrolled
# Scrolling using mouse wheel
bind . <Button-5> [list .sf yview scroll 3 units]
bind . <Button-4> [list .sf yview scroll -3 units]
pack .sby -side right -fill y
pack .sf -side right -expand yes -fill both
bind . <Escape> { saveSettings ; exit 0 }
bind . <Key-F1> { appHelp $readmeFile }
wm protocol . WM_DELETE_WINDOW { saveSettings ; exit 0 }
switchCard lastUsed

if {$saveWindow && [string length $mainWinGeometry] && $argc == 0} { wm geometry . $mainWinGeometry }

